package com.zanxiang.manage.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.zanxiang.manage.domain.bo.PayApplicationBO;
import com.zanxiang.manage.domain.dto.PayApplicationDTO;
import com.zanxiang.manage.domain.params.PayApplicationAddUpdateParam;
import com.zanxiang.manage.domain.params.PayApplicationListNoPageParam;
import com.zanxiang.manage.domain.params.PayApplicationListParam;
import com.zanxiang.manage.domain.vo.PayApplicationChoiceVO;
import com.zanxiang.manage.domain.vo.PayApplicationVO;
import com.zanxiang.mybatis.entity.PayApplication;

import java.util.List;
import java.util.Map;

/**
 * 游戏应用表 服务类接口
 *
 * @author xufeng
 * @date 2022-06-20 15:16
 */
public interface PayApplicationService extends IService<PayApplication> {

    /**
     * 支付应用程序地图
     *
     * @return {@link Map}<{@link Long}, {@link PayApplicationDTO}>
     */
    Map<Long, PayApplicationDTO> payApplicationMap();

    /**
     * 支付应用情况
     *
     * @param payApplicationId   : 支付应用程序id
     * @param payApplicationType : 支付应用程序类型
     * @param appId              : 应用程序id
     * @param company            : 公司
     * @return {@link Map}<{@link Long}, {@link PayApplicationDTO}>
     */
    Map<Long, PayApplicationDTO> payApplicationCondition(Integer payApplicationId, Integer payApplicationType, String appId, String company);

    /**
     * 支付应用选择列表
     *
     * @return : 返回所有支付应用信息
     */
    List<PayApplicationChoiceVO> payApplicationChoiceList();

    /**
     * 支付应用添加或者更新
     *
     * @param param : 参数
     * @return : 返回结果
     */
    Boolean addOrUpdate(PayApplicationAddUpdateParam param);

    /**
     * 根据主键更新状态
     *
     * @param id     : 主键id
     * @param status : 状态
     * @return : 返回更新结果
     */
    Boolean statusUpdate(Long id, Integer status);

    /**
     * 根据id新增与更新操作
     *
     * @param bo
     * @return
     */
    Boolean saveOrUpdate(PayApplicationBO bo);

    /**
     * shang
     *
     * @param bo
     * @return
     */
    IPage<PayApplicationVO> list(PayApplicationListParam bo);

    /**
     * shang
     *
     * @param bo
     * @return
     */
    List<PayApplicationVO> listNoPage(PayApplicationListNoPageParam bo);

    /**
     * 获取游戏应用二维码图片地址
     *
     * @param id : 应用id
     * @return : 返回游戏二维码
     */
    String getQrCode(Long id);

    /**
     * 删除
     *
     * @param id : 要删除的信息id
     * @return : 返回删除结果
     */
    boolean deleteById(Long id);
}
