package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zanxiang.common.enums.CpStatusEnum;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.DateUtils;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.URIUtil;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.mybatis.entity.*;
import com.zanxiang.sdk.domain.dto.GamePayWayDTO;
import com.zanxiang.sdk.domain.dto.PlatformOrderDTO;
import com.zanxiang.sdk.domain.result.PushCpResult;
import com.zanxiang.sdk.service.*;
import com.zanxiang.sdk.util.SignUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 订单履约逻辑
 *
 * @author xufeng
 * @date 2022/6/27 11:17
 */
@Slf4j
@Service
@Transactional(rollbackFor = {RuntimeException.class, Exception.class})
public class PerformOrderServiceImpl implements PerformOrderService {

    @Autowired
    private OrderService orderService;

    @Autowired
    private LogPayCpService logPayCpService;

    @Autowired
    private GameUserRoleService gameUserRoleService;

    @Autowired
    private GameUserService gameUserService;

    @Autowired
    private UserService userService;

    @Autowired
    private GamePayWayService gamePayWayService;

    @Autowired
    private PayMerchantService payMerchantService;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private GameExtService gameExtService;

    @Override
    public Boolean pushCp(PlatformOrderDTO orderInfo) {
        log.info("充值回调CP, orderInfo : {} ----------start---------", JsonUtil.toString(orderInfo));
        if (Objects.equals(orderInfo.getCpStatus(), CpStatusEnum.SUCCESS.getStatus())) {
            return true;
        }
        GameExt gameExt = gameExtService.getByGameId(orderInfo.getGameId());
        String cpPaybackUrl = gameExt.getPayCallBackUrl();
        //回调CP
        Map<String, String> map = new HashMap<>(9);
        map.put("userId", String.valueOf(orderInfo.getUserId()));
        map.put("money", orderInfo.getAmount().stripTrailingZeros().toPlainString());
        map.put("time", String.valueOf(System.currentTimeMillis()));
        map.put("serverId", orderInfo.getServerId());
        map.put("roleId", orderInfo.getRoleId());
        map.put("roleName", orderInfo.getRoleName());
        map.put("orderId", orderInfo.getCpOrderId());
        map.put("ext", URIUtil.encodeURIComponent(orderInfo.getExt()));
        map.put("sign", this.paySign(orderInfo.getGameId(), map));
        //url
        String url = URIUtil.fillUrlParams(cpPaybackUrl, map, false);
        String resultStr;
        log.error("通知CP支付回调, url : {}", url);
        try {
            resultStr = restTemplate.getForObject(url, String.class);
        } catch (Exception e) {
            log.error("回调CP充值接口异常, orderInfo : {}, e : {}", JsonUtil.toString(orderInfo), e.getMessage());
            throw new BaseException("回调CP充值接口异常");
        }
        log.error("通知CP支付回调结果, resultStr : {}", resultStr);
        PushCpResult pushCpResult = JsonUtil.toObj(resultStr, PushCpResult.class);
        boolean pushSuccess = Objects.equals(pushCpResult.getRet(), "200");
        //更新订单信息
        orderService.update(new LambdaUpdateWrapper<Order>()
                .set(Order::getCpNotifyCnt, orderInfo.getCpNotifyCnt() + 1)
                .set(Order::getLastCpNotifyTime, LocalDateTime.now())
                .set(Order::getCpStatus, pushSuccess ? CpStatusEnum.SUCCESS.getStatus() : CpStatusEnum.FAIL.getStatus())
                .set(pushSuccess, Order::getEndTime, LocalDateTime.now())
                .eq(Order::getOrderId, orderInfo.getOrderId())
        );
        //更新回调记录
        LogPayCp logPayCp = new LogPayCp();
        logPayCp.setOrderId(orderInfo.getOrderId());
        logPayCp.setCpOrderId(orderInfo.getCpOrderId());
        logPayCp.setStatus(orderInfo.getStatus());
        logPayCp.setCpStatus(pushSuccess ? CpStatusEnum.SUCCESS.getStatus() : CpStatusEnum.FAIL.getStatus());
        logPayCp.setCpPaybackUrl(cpPaybackUrl);
        logPayCp.setParams(JsonUtil.toString(map));
        logPayCp.setExt(orderInfo.getExt());
        logPayCp.setNotifyCnt(orderInfo.getCpNotifyCnt() + 1);
        logPayCp.setCreateTime(LocalDateTime.now());
        logPayCp.setUpdateTime(LocalDateTime.now());
        logPayCpService.save(logPayCp);
        log.info("充值回调CP----------end---------");
        return pushSuccess;
    }

    private String paySign(Long gameId, Map<String, String> map) {
        //查询游戏
        GameExt gameExt = gameExtService.getByGameId(gameId);
        if (Strings.isBlank(gameExt.getPayKey())) {
            return null;
        }
        StringBuilder sb = new StringBuilder();
        sb.append("payKey=").append(gameExt.getPayKey());
        sb.append("&userId=").append(map.get("userId"));
        sb.append("&money=").append(map.get("money"));
        sb.append("&time=").append(map.get("time"));
        sb.append("&serverId=").append(map.get("serverId"));
        sb.append("&orderId=").append(map.get("orderId"));
        //加密
        String sign;
        try {
            sign = SignUtil.MD5(sb.toString()).toUpperCase();
        } catch (Exception e) {
            throw new BaseException("支付加密异常");
        }
        return sign;
    }

    @Override
    public Boolean checkIsFirstRecharge(PlatformOrderDTO orderInfo) {
        log.info("订单:{} 检查更新用户是否为首充 ----------start---------", orderInfo.getOrderId());
        try {
            Order one = orderService.getOne(new QueryWrapper<Order>().lambda()
                    .eq(Order::getUserId, orderInfo.getUserId())
                    .eq(Order::getStatus, OrderStateEnum.SUCCESS.getCode())
                    .ne(Order::getOrderId, orderInfo.getOrderId()).last("limit 1"));
            if (Objects.isNull(one)) {
                Order order = new Order();
                order.setOrderId(order.getOrderId());
                order.setIsFirstRecharge(1);
                orderService.updateById(order);
                log.info("订单:{} 验证订单是否为首充成功", orderInfo.getOrderId());
                return true;
            }
        } catch (Exception e) {
            log.error("订单:{} 验证订单是否为首充失败 e:{}", orderInfo.getOrderId(), e);
        }
        return true;
    }

    @Override
    public Boolean userRechargeTotal(PlatformOrderDTO orderInfo) {
        log.info("订单:{} 更新用户充值统计 ----------start---------", orderInfo.getOrderId());
        try {
            //订单取消情况 做减法
            float amount = orderInfo.getAmount().floatValue();
            int num = 1;
            if (orderInfo.getStatus().equals(OrderStateEnum.CANCEL.getCode())) {
                amount = -amount;
                num = -1;
            }
            //游戏用户角色统计更新
            if (StringUtils.isNotEmpty(orderInfo.getRoleId()) && !"0".equals(orderInfo.getRoleId())) {
                GameUserRole gameUserRole = new GameUserRole();
                gameUserRole.setRoleId(orderInfo.getRoleId());
                gameUserRoleService.update(gameUserRole, new UpdateWrapper<GameUserRole>().lambda()
                        .set(GameUserRole::getLastRechargeTime, orderInfo.getPayTime())
                        .setSql("recharge_count=recharge_count+" + num)
                        .setSql("recharge_money=recharge_money+" + amount)
                        .setEntity(gameUserRole)
                );
            } else {
                log.info("订单:{} 用户玩家角色RoleId为空，不进行gameUserRole充值统计汇总", orderInfo.getOrderId());
            }
            //游戏用户统计更新
            if (orderInfo.getMgUserId() != null) {
                GameUser gameUser = new GameUser();
                gameUser.setId(orderInfo.getMgUserId());
                gameUserService.update(gameUser, new UpdateWrapper<GameUser>().lambda()
                        .set(GameUser::getLastRechargeTime, orderInfo.getPayTime())
                        .setSql("recharge_count=recharge_count+" + num)
                        .setSql("recharge_money=recharge_money+" + amount)
                        .setEntity(gameUser)
                );
            } else {
                log.info("订单:{} 用户玩家MgUserId为空，不进行gameUser充值统计汇总", orderInfo.getOrderId());
            }
            //用户统计更新
            if (orderInfo.getUserId() != null) {
                User user = new User();
                user.setId(orderInfo.getUserId());
                userService.update(user, new UpdateWrapper<User>().lambda()
                        .set(User::getLastRechargeTime, orderInfo.getPayTime())
                        .setSql("recharge_count=recharge_count+" + num)
                        .setSql("recharge_money=recharge_money+" + amount)
                        .setEntity(user)
                );
            } else {
                log.info("订单:{} 用户UserId为空，不进行User充值统计汇总", orderInfo.getOrderId());
            }
            return true;
        } catch (Exception e) {
            log.error("订单:{} 用户充值统计逻辑操作失败 e:{}", orderInfo.getOrderId(), e);
        }
        return false;
    }

    @Override
    public Boolean payMerchantTotal(PlatformOrderDTO orderInfo) {
        log.info("订单:{} 更新商户号统计 ----------start---------", orderInfo.getOrderId());
        try {
            if (orderInfo.getGamePaywayId() == null) {
                log.error("订单:{} 更新商户号统计时，该订单GamePaywayId为空", orderInfo.getOrderId());
                return false;
            }
//            GamePayWay gamePayWayInfo = gamePayWayService.getInfo(String.valueOf(orderInfo.getGamePaywayId()));


            GamePayWayDTO gamePayWayDTO = gamePayWayService.getById(orderInfo.getGamePaywayId());

            if (Objects.isNull(gamePayWayDTO)) {
                log.error("订单:{} 更新商户号统计时，获取GamePayway表id:{}信息为空", orderInfo.getOrderId(), orderInfo.getGamePaywayId());
                return false;
            }
            //todo : 商户统计要修改
//            if (gamePayWayInfo.getPayMerchantId() <= 0) {
//                log.error("订单:{} 更新商户号统计时，获取GamePayway表id:{}信息中PayMerchantId为空", orderInfo.getOrderId(), orderInfo.getGamePaywayId());
//                return false;
//            }
//            PayMerchant payMerchantInfo = payMerchantService.getById(gamePayWayInfo.getPayMerchantId());
//            if (Objects.isNull(payMerchantInfo)) {
//                log.error("订单:{} 更新商户号统计时，获取PayMerchant表id:{}信息为空", orderInfo.getOrderId(), gamePayWayInfo.getPayMerchantId());
//                return false;
//            }
            PayMerchant payMerchantInfo = payMerchantService.getOne(new LambdaQueryWrapper<PayMerchant>()
                    .eq(PayMerchant::getMerchantNo, gamePayWayDTO.getMerchantNo()));

            BigDecimal amount = orderInfo.getAmount();
            LocalDateTime updateTime = payMerchantInfo.getDayUpdateDate();
            Date date = new Date();
            String totalPayAmount, mouthPayAmount, weakerPayAmount, dayPayAmount;
            totalPayAmount = " total_pay_amount = total_pay_amount + " + amount;
            mouthPayAmount = " mouth_pay_amount = mouth_pay_amount + " + amount;
            weakerPayAmount = " weaker_pay_amount = weaker_pay_amount + " + amount;
            dayPayAmount = " day_pay_amount = day_pay_amount + " + amount;
            if (updateTime != null) {
                if (!DateUtils.isThisMonth(payMerchantInfo.getMouthUpdateDate())) {
                    mouthPayAmount = " mouth_pay_amount = " + amount;
                }
                if (!DateUtils.isThisWeek(payMerchantInfo.getWeakerUpdateDate())) {
                    weakerPayAmount = " weaker_pay_amount = " + amount;
                }
                if (!DateUtils.isToday(payMerchantInfo.getDayUpdateDate())) {
                    dayPayAmount = " day_pay_amount = " + amount;
                }
            }
            payMerchantService.update(new UpdateWrapper<PayMerchant>().lambda()
                    .set(PayMerchant::getMouthUpdateDate, date)
                    .set(PayMerchant::getWeakerUpdateDate, date)
                    .set(PayMerchant::getDayUpdateDate, date)
                    .setSql(totalPayAmount)
                    .setSql(mouthPayAmount)
                    .setSql(weakerPayAmount)
                    .setSql(dayPayAmount)
                    .setEntity(payMerchantInfo)
            );
            log.info("订单:{} 更新商户号统计 ----------end---------", orderInfo.getOrderId());
            return true;
        } catch (Exception e) {
            log.error("订单:{} 更新商户号统计失败 e:{}", orderInfo.getOrderId(), e);
        }
        return false;
    }
}
